@@ -60,16 +60,16 @@ class SignatureConfiguration:
     If set to a non-integer, any number of references is accepted (otherwise a mismatch raises an error).
     """
 
-    signature_methods: FrozenSet[SignatureMethod] = frozenset()
+    signature_methods: FrozenSet[SignatureMethod] = frozenset(sm for sm in SignatureMethod if "SHA1" not in sm.name)
     """
     Set of acceptable signature methods (signature algorithms). Any signature generated using an algorithm not listed
-    here will fail verification (but if this set is left empty, then all supported algorithms are accepted).
+    here will fail verification.
     """
 
-    digest_algorithms: FrozenSet[DigestAlgorithm] = frozenset()
+    digest_algorithms: FrozenSet[DigestAlgorithm] = frozenset(da for da in DigestAlgorithm if "SHA1" not in da.name)
     """
     Set of acceptable digest algorithms. Any signature or reference transform generated using an algorithm not listed
-    here will cause verification to fail (but if this set is left empty, then all supported algorithms are accepted).
+    here will cause verification to fail.
     """
 
     ignore_ambiguous_key_info: bool = False
@@ -107,6 +107,8 @@ class XMLVerifier(XMLSignatureProcessor):
     Create a new XML Signature Verifier object, which can be used to verify multiple pieces of data.
     """
 
+    _default_reference_c14n_method = CanonicalizationMethod.CANONICAL_XML_1_0
+
     def _get_signature(self, root):
         if root.tag == ds_tag("Signature"):
             return root
@@ -188,7 +190,7 @@ class XMLVerifier(XMLSignatureProcessor):
         else:
             return inclusive_namespaces.get("PrefixList").split(" ")
 
-    def _apply_transforms(self, payload, transforms_node, signature, c14n_algorithm: CanonicalizationMethod):
+    def _apply_transforms(self, payload, *, transforms_node: etree._Element, signature: etree._Element):
         transforms, c14n_applied = [], False
         if transforms_node is not None:
             transforms = self._findall(transforms_node, "Transform")
@@ -214,7 +216,7 @@ class XMLVerifier(XMLSignatureProcessor):
             c14n_applied = True
 
         if not c14n_applied and not isinstance(payload, (str, bytes)):
-            payload = self._c14n(payload, algorithm=c14n_algorithm)
+            payload = self._c14n(payload, algorithm=self._default_reference_c14n_method)
 
         return payload
 
@@ -338,7 +340,7 @@ class XMLVerifier(XMLSignatureProcessor):
         signature_method = self._find(signed_info, "SignatureMethod")
         signature_value = self._find(signature, "SignatureValue")
         signature_alg = SignatureMethod(signature_method.get("Algorithm"))
-        if self.config.signature_methods and signature_alg not in self.config.signature_methods:
+        if signature_alg not in self.config.signature_methods:
             raise InvalidInput(f"Signature method {signature_alg.name} forbidden by configuration")
         raw_signature = b64decode(signature_value.text)
         x509_data = signature.find("ds:KeyInfo/ds:X509Data", namespaces=namespaces)
@@ -384,6 +386,7 @@ class XMLVerifier(XMLSignatureProcessor):
 
             if signature_alg.name.startswith("ECDSA"):
                 raw_signature = self._encode_dss_signature(raw_signature, signing_cert.get_pubkey().bits())
+
             try:
                 digest_alg_name = str(digest_algorithm_implementations[signature_alg].name)
                 openssl_verify(signing_cert, raw_signature, signed_info_c14n, digest_alg_name)
@@ -463,10 +466,9 @@ class XMLVerifier(XMLSignatureProcessor):
         digest_method_alg_name = self._find(reference, "DigestMethod").get("Algorithm")
         digest_value = self._find(reference, "DigestValue")
         payload = self._resolve_reference(copied_root, reference, uri_resolver=uri_resolver)
-        # TODO: payload-specific c14n alg
-        payload_c14n = self._apply_transforms(payload, transforms, copied_signature_ref, c14n_algorithm)
+        payload_c14n = self._apply_transforms(payload, transforms_node=transforms, signature=copied_signature_ref)
         digest_alg = DigestAlgorithm(digest_method_alg_name)
-        if self.config.digest_algorithms and digest_alg not in self.config.digest_algorithms:
+        if digest_alg not in self.config.digest_algorithms:
             raise InvalidInput(f"Digest algorithm {digest_alg.name} forbidden by configuration")
 
         if b64decode(digest_value.text) != self._get_digest(payload_c14n, digest_alg):
