@@ -6,6 +6,7 @@ import re
 import sys
 import unittest
 from base64 import b64decode, b64encode
+from concurrent.futures import ThreadPoolExecutor
 from dataclasses import replace
 from glob import glob
 from xml.etree import ElementTree as stdlibElementTree
@@ -33,8 +34,10 @@ from signxml import (  # noqa:E402
     methods,
     namespaces,
 )
+from signxml.util import ds_tag  # noqa:E402
 from signxml.xades import (  # noqa:E402
     XAdESDataObjectFormat,
+    XAdESSignatureConfiguration,
     XAdESSignaturePolicy,
     XAdESSigner,
     XAdESVerifier,
@@ -42,6 +45,17 @@ from signxml.xades import (  # noqa:E402
 )
 
 
+class XMLSignerWithSHA1(XMLSigner):
+    def check_deprecated_methods(self):
+        pass
+
+
+sha1_ok = SignatureConfiguration(signature_methods=list(SignatureMethod), digest_algorithms=list(DigestAlgorithm))
+xades_sha1_ok = XAdESSignatureConfiguration(
+    signature_methods=list(SignatureMethod), digest_algorithms=list(DigestAlgorithm)
+)
+
+
 def reset_tree(t, method):
     if not isinstance(t, str):
         for s in t.findall(".//ds:Signature", namespaces=namespaces):
@@ -101,7 +115,7 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
         self.assertEqual(SignatureMethod.RSA_SHA256.value, "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256")
         self.assertEqual(SignatureConstructionMethod.enveloped, methods.enveloped)
         with self.assertRaisesRegex(InvalidInput, "Unknown signature construction method"):
-            signer = XMLSigner(method=None)
+            XMLSignerWithSHA1(method=None)
 
         with self.assertRaisesRegex(InvalidInput, "must be an XML element"):
             XMLSigner(signature_algorithm="hmac-sha256").sign("x", key=b"abc")
@@ -113,8 +127,8 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
             "http://www.w3.org/2001/10/xml-exc-c14n#WithComments",
         }
 
-        all_methods = itertools.product(digest_algs, sig_algs, methods, c14n_algs)
-        for digest_alg, sig_alg, method, c14n_alg in all_methods:
+        def test_case(case):
+            digest_alg, sig_alg, method, c14n_alg = case
             data = [etree.parse(f).getroot() for f in self.example_xml_files]
             # FIXME: data.extend(stdlibElementTree.parse(f).getroot() for f in (self.example_xml_files)
             data.append(stdlibElementTree.parse(self.example_xml_files[0]).getroot())
@@ -124,7 +138,7 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
                     continue
                 print(digest_alg.name, sig_alg.name, c14n_alg, method, type(d))
                 reset_tree(d, method)
-                signer = XMLSigner(
+                signer = XMLSignerWithSHA1(
                     method=method,
                     signature_algorithm=sig_alg,
                     digest_algorithm=digest_alg,
@@ -136,7 +150,7 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
                 )
                 # print(etree.tostring(signed))
                 hmac_key = self.keys["hmac"] if sig_alg_type == "hmac" else None
-                verify_kwargs = dict(require_x509=False, hmac_key=hmac_key, validate_schema=True)
+                verify_kwargs = dict(require_x509=False, hmac_key=hmac_key, validate_schema=True, expect_config=sha1_ok)
 
                 if method == methods.detached:
 
@@ -169,7 +183,12 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
                         XMLVerifier().verify(signed_data, id_attribute="X", **verify_kwargs)
 
                 with self.assertRaisesRegex(InvalidInput, "Expected a X.509 certificate based signature"):
-                    XMLVerifier().verify(signed_data, hmac_key=hmac_key, uri_resolver=verify_kwargs.get("uri_resolver"))
+                    XMLVerifier().verify(
+                        signed_data,
+                        hmac_key=hmac_key,
+                        uri_resolver=verify_kwargs.get("uri_resolver"),
+                        expect_config=sha1_ok,
+                    )
 
                 if method != methods.detached:
                     with self.assertRaisesRegex(InvalidSignature, "Digest mismatch"):
@@ -197,6 +216,10 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
                         verify_kwargs["hmac_key"] = b"SECRET"
                         XMLVerifier().verify(signed_data, **verify_kwargs)
 
+        executor = ThreadPoolExecutor()
+        for _ in executor.map(test_case, itertools.product(digest_algs, sig_algs, methods, c14n_algs)):
+            pass
+
     def test_x509_certs(self):
         from OpenSSL.crypto import FILETYPE_PEM
         from OpenSSL.crypto import Error as OpenSSLCryptoError
@@ -205,28 +228,26 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
         tree = etree.parse(self.example_xml_files[0])
         ca_pem_file = os.path.join(os.path.dirname(__file__), "example-ca.pem").encode("utf-8")
         crt, key = self.load_example_keys()
-        for hash_alg in "sha1", "sha256":
-            for method in methods.enveloped, methods.enveloping:
-                print(hash_alg, method)
-                data = tree.getroot()
-                reset_tree(data, method)
-                signer = XMLSigner(method=method, signature_algorithm="rsa-" + hash_alg)
-                signed = signer.sign(data, key=key, cert=crt)
-                signed_data = etree.tostring(signed)
-                XMLVerifier().verify(signed_data, ca_pem_file=ca_pem_file)
-                XMLVerifier().verify(signed_data, x509_cert=crt)
-                XMLVerifier().verify(signed_data, x509_cert=load_certificate(FILETYPE_PEM, crt))
-                XMLVerifier().verify(signed_data, x509_cert=crt, cert_subject_name="*.example.com")
+        for method in methods.enveloped, methods.enveloping:
+            data = tree.getroot()
+            reset_tree(data, method)
+            signer = XMLSigner(method=method, signature_algorithm=SignatureMethod.RSA_SHA256)
+            signed = signer.sign(data, key=key, cert=crt)
+            signed_data = etree.tostring(signed)
+            XMLVerifier().verify(signed_data, ca_pem_file=ca_pem_file)
+            XMLVerifier().verify(signed_data, x509_cert=crt)
+            XMLVerifier().verify(signed_data, x509_cert=load_certificate(FILETYPE_PEM, crt))
+            XMLVerifier().verify(signed_data, x509_cert=crt, cert_subject_name="*.example.com")
 
-                with self.assertRaises(OpenSSLCryptoError):
-                    XMLVerifier().verify(signed_data, x509_cert=crt[::-1])
+            with self.assertRaises(OpenSSLCryptoError):
+                XMLVerifier().verify(signed_data, x509_cert=crt[::-1])
 
-                with self.assertRaises(InvalidSignature):
-                    XMLVerifier().verify(signed_data, x509_cert=crt, cert_subject_name="test")
+            with self.assertRaises(InvalidSignature):
+                XMLVerifier().verify(signed_data, x509_cert=crt, cert_subject_name="test")
 
-                with self.assertRaisesRegex(InvalidCertificate, "unable to get local issuer certificate"):
-                    XMLVerifier().verify(signed_data)
-                # TODO: negative: verify with wrong cert, wrong CA
+            with self.assertRaisesRegex(InvalidCertificate, "unable to get local issuer certificate"):
+                XMLVerifier().verify(signed_data)
+            # TODO: negative: verify with wrong cert, wrong CA
 
     def test_xmldsig_interop_examples(self):
         ca_pem_file = os.path.join(os.path.dirname(__file__), "interop", "cacert.pem").encode("utf-8")
@@ -234,7 +255,7 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
             print("Verifying", signature_file)
             with open(signature_file, "rb") as fh:
                 with self.assertRaisesRegex(InvalidCertificate, "certificate has expired"):
-                    XMLVerifier().verify(fh.read(), ca_pem_file=ca_pem_file)
+                    XMLVerifier().verify(fh.read(), ca_pem_file=ca_pem_file, expect_config=sha1_ok)
 
     def test_xmldsig_interop_TR2012(self):
         def get_x509_cert(**kwargs):
@@ -256,6 +277,7 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
                         hmac_key="testkey",
                         validate_schema=True,
                         cert_resolver=get_x509_cert if "x509digest" in signature_file else None,
+                        expect_config=sha1_ok,
                     )
                     sig.decode("utf-8")
                 except Exception as e:
@@ -325,6 +347,7 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
                         x509_cert=get_x509_cert(signature_file),
                         cert_resolver=cert_resolver if "issuer-serial" in signature_file else None,
                         ca_pem_file=get_ca_pem_file(signature_file),
+                        expect_config=sha1_ok,
                     )
                     decoded_sig = sig.decode("utf-8")
                     if "HMACOutputLength" in decoded_sig or "bad" in signature_file or "expired" in signature_file:
@@ -535,6 +558,20 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
             "8GPAVJstDxHyuoJqec8C0ssji4zfdXanu1YHGlWbfx0=",
         )
 
+        # Test correct default c14n method for payload when c14n transform metadata is omitted
+        def _build_transforms_for_reference(transforms_node, reference):
+            etree.SubElement(
+                transforms_node, ds_tag("Transform"), Algorithm=SignatureConstructionMethod.enveloped.value
+            )
+
+        signer._build_transforms_for_reference = _build_transforms_for_reference
+        signed3 = signer.sign(**sign_args)
+        self.assertEqual(
+            signed3.find(".//ds:SignatureValue", namespaces=namespaces).text,
+            "/iezjApGBVMMUspj5WyZwIOEw30qLX3Gv576vwFMAbQ=",
+        )
+        XMLVerifier().verify(signed3, hmac_key=b"secret", require_x509=False)
+
     def test_excision_of_untrusted_comments(self):
         pass  # TODO: test comments excision
 
@@ -543,13 +580,13 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
         with open(os.path.join(wsse_dir, "examples", "server_public.pem"), "rb") as fh:
             crt = fh.read()
         data = etree.parse(os.path.join(wsse_dir, "test", "unit", "client", "files", "valid wss resp.xml"))
-        XMLVerifier().verify(data, x509_cert=crt, validate_schema=False, expect_references=2)
+        XMLVerifier().verify(data, x509_cert=crt, validate_schema=False, expect_references=2, expect_config=sha1_ok)
 
         data = etree.parse(
             os.path.join(wsse_dir, "test", "unit", "client", "files", "invalid wss resp - changed content.xml")
         )
         with self.assertRaisesRegex(InvalidDigest, "Digest mismatch for reference 0"):
-            XMLVerifier().verify(data, x509_cert=crt, validate_schema=False, expect_references=2)
+            XMLVerifier().verify(data, x509_cert=crt, validate_schema=False, expect_references=2, expect_config=sha1_ok)
 
     def test_psha1(self):
         from signxml.util import p_sha1
@@ -613,12 +650,27 @@ class TestSignXML(unittest.TestCase, LoadExampleKeys):
         config = SignatureConfiguration(location="./foo/bar/")
         with self.assertRaisesRegex(InvalidInput, "Expected to find XML element Signature in data"):
             verifier.verify(signed, x509_cert=cert, expect_config=config)
-        config = SignatureConfiguration(signature_methods=[SignatureMethod.ECDSA_SHA384])
+        config = SignatureConfiguration(signature_methods=[])
         with self.assertRaisesRegex(InvalidInput, "Signature method RSA_SHA256 forbidden by configuration"):
             verifier.verify(signed, x509_cert=cert, expect_config=config)
         config = SignatureConfiguration(digest_algorithms=[DigestAlgorithm.SHA3_512])
         with self.assertRaisesRegex(InvalidInput, "Digest algorithm SHA256 forbidden by configuration"):
             verifier.verify(signed, x509_cert=cert, expect_config=config)
+        config = SignatureConfiguration(digest_algorithms=[])
+        with self.assertRaisesRegex(InvalidInput, "Digest algorithm SHA256 forbidden by configuration"):
+            verifier.verify(signed, x509_cert=cert, expect_config=config)
+
+    def test_sha1_policy(self):
+        data = etree.parse(self.example_xml_files[0]).getroot()
+        cert, key = self.load_example_keys()
+        with self.assertRaisesRegex(InvalidInput, "SHA1-based algorithms are not supported"):
+            XMLSigner(signature_algorithm=SignatureMethod.RSA_SHA1)
+        signer = XMLSignerWithSHA1(signature_algorithm=SignatureMethod.RSA_SHA1, digest_algorithm=DigestAlgorithm.SHA1)
+        signed = signer.sign(data, cert=cert, key=key)
+        verifier = XMLVerifier()
+        with self.assertRaisesRegex(InvalidInput, "Signature method RSA_SHA1 forbidden by configuration"):
+            verifier.verify(signed, x509_cert=cert)
+        verifier.verify(signed, x509_cert=cert, expect_config=sha1_ok)
 
 
 class TestXAdES(unittest.TestCase, LoadExampleKeys):
@@ -658,6 +710,11 @@ class TestXAdES(unittest.TestCase, LoadExampleKeys):
             data_object_format=self.data_object_format,
         )
         signed_doc = signer.sign(doc, key=key, cert=cert)
+
+        for ref in signed_doc.findall(".//{http://www.w3.org/2000/09/xmldsig#}Reference"):
+            if "SignedProperties" in ref.get("URI"):
+                self.assertEqual(ref.get("Type"), "http://uri.etsi.org/01903#SignedProperties")
+
         verifier = XAdESVerifier()
         verify_results = verifier.verify(
             signed_doc, x509_cert=cert, expect_references=3, expect_signature_policy=self.signature_policy
@@ -681,7 +738,11 @@ class TestXAdES(unittest.TestCase, LoadExampleKeys):
             with open(sig_file, "rb") as fh:
                 doc = etree.parse(fh)
             cert = doc.find("//{http://www.w3.org/2000/09/xmldsig#}X509Certificate").text
-            kwargs = dict(x509_cert=cert, expect_references=self.expect_references.get(os.path.basename(sig_file), 2))
+            kwargs = dict(
+                x509_cert=cert,
+                expect_references=self.expect_references.get(os.path.basename(sig_file), 2),
+                expect_config=xades_sha1_ok,
+            )
             if "nonconformant" in sig_file:
                 kwargs.update(validate_schema=False)
             if "sigPolStore" in sig_file:
